﻿using C1.Win.C1FlexGrid;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Json;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace SparklinesInFlexGridSample
{
    public partial class Form1 : Form
    {
        IEnumerable<dynamic> aggregatedData;
        public Form1()
        {
            InitializeComponent();
            this.Load += Form1_Load;
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            //Creates data for the FlexGrid
            CreateData();
            //Sets the relevant grid properties for showing sparklines in grid.
            SetUpGrid();
        }

        private void CreateData()
        {
            List<StockData> stockData = new List<StockData>();

            string json;
            using (StreamReader r = new StreamReader(Path.Combine(Environment.CurrentDirectory, "..\\..\\CompaniesStockData.json")))
            {
                json = r.ReadToEnd();
            }
            MemoryStream ms = new MemoryStream(Encoding.Unicode.GetBytes(json));
            DataContractJsonSerializer ser = new DataContractJsonSerializer(stockData.GetType(),
                 new DataContractJsonSerializerSettings
                 {
                     DateTimeFormat = new DateTimeFormat("dd-MM-yyyy")
                 });
            stockData = ser.ReadObject(ms) as List<StockData>;
            ms.Close();

            DateTime latestWeekEndDate = stockData.First().VolValues.Max(y => y.Date);
            DateTime latestWeekStartDate = latestWeekEndDate.AddDays(-6);

            DateTime priorWeekStartDate = latestWeekEndDate.AddDays(-13);
            DateTime priorWeekEndDate = latestWeekStartDate.AddDays(-1);

            aggregatedData = from data in stockData
                             let latestWeekVolume = data.VolValues.Where(x => x.Date <= latestWeekEndDate && x.Date >= latestWeekStartDate).Sum(x=>x.Volume)
                             let priorWeekVolume = data.VolValues.Where(x => x.Date <= priorWeekEndDate && x.Date >= priorWeekStartDate).Sum(x => x.Volume)
                             let change = Math.Abs(latestWeekVolume - priorWeekVolume)
                             select new AggregatedStockData
                             {
                                 Company = (Image)Properties.Resources.ResourceManager.GetObject(data.CompanyName),
                                 LatestWeekVolume = latestWeekVolume,
                                 PriorWeekVolume = priorWeekVolume,
                                 Change = change,
                                 PercentageChange = (change / (double)priorWeekVolume),
                                 VolumeValues = data.VolValues.Select(x=>x.Volume).ToList<int>()
                             };
        }

        
        private void SetUpGrid()
        {
            _flex.DataSource = aggregatedData.ToList();

            Column volumeTrendCol = _flex.Cols["VolumeValues"];
            volumeTrendCol.Caption = "Volume Trend Last 31 days\n as of 31-10-2018";
            //Set the Column's ShowSparkline property to true to show sparklines in that column
            volumeTrendCol.ShowSparkline = true;
            //Sets the type of Sparkline
            volumeTrendCol.Sparkline.SparklineType = SparklineType.Line;
            //Highlight the highest and the lowest data points
            volumeTrendCol.Sparkline.ShowHigh = true;
            volumeTrendCol.Sparkline.ShowLow = true;
            //Styling the Sparkline
            volumeTrendCol.Sparkline.Styles.HighMarkerColor = Color.Green;
            volumeTrendCol.Sparkline.Styles.LowMarkerColor = Color.Red;

            //Setting additional grid properties
            _flex.Rows.DefaultSize = 60;
            _flex.Cols.DefaultSize = 120;           
            volumeTrendCol.StarWidth = "*";
            volumeTrendCol.StyleNew.Margins = new System.Drawing.Printing.Margins(10, 10, 5, 5);
            _flex.Rows[0].StyleNew.Font = new Font(FontFamily.GenericSansSerif, 10f, FontStyle.Bold);
            for (int i = 0; i < _flex.Cols.Count; i++)
            {
                _flex.Cols[i].TextAlign = TextAlignEnum.CenterCenter;
                _flex.Cols[i].TextAlignFixed = TextAlignEnum.CenterCenter;
                _flex.Cols[i].Format = "N0";
            }
            _flex.Cols["PercentageChange"].Format = "P2";
            _flex.Cols["LatestWeekVolume"].Caption = "LatestWeek\nVolume";
            _flex.Cols["PriorWeekVolume"].Caption = "PriorWeek\nVolume";
            _flex.Cols["PercentageChange"].Caption = "% Change";
        }
    }

}
